/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.ars.ars.mgmt.commands;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.ars.ars.mgmt.datatree.DataIdentNode;
import de.bsvrz.ars.ars.mgmt.datatree.DataIdentTree;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.losb.util.Util;
import de.bsvrz.sys.funclib.losb.util.cmdinterface.Command;

import java.util.*;

/**
 * Kommando zum Ausgeben von Informationen einer bestimmten Datenidentifikation aus dem {@link de.bsvrz.ars.ars.mgmt.datatree.DataIdentTree}
 *
 * @author beck et al. projects GmbH
 * @author Thomas Mueller
 * @version $Revision: 6419 $ / $Date: 2009-03-10 23:15:57 +0100 (Di, 10 Mrz 2009) $ / ($Author: rs $)
 */
public class DataIdentInfo extends Command {

	private static String PAGER = " (vor: [enter], zurueck: <[enter])";

	private ArchiveManager archMgr;

	private DataIdentTree tree;

	private DataModel model;

	private boolean stopCmd;

	private final int linesPerPage = 40;

	private final int maxPerLine = 77;

	private SystemObject[] objects = null;

	private int obj = -1;

	private AttributeGroup[] atgs = null;

	private int atg = -1;

	private Aspect[] asps = null;

	private int asp = -1;

	private int[] simVars = null;

	private int simVar = -2;

	/** simple constructor */
	public DataIdentInfo(ArchiveManager am) {
		super("Informationen zu einer Datenidentifikation", "Detailierte Informationen zu einer bestimmten Datenidentifikation abrufen.");
		archMgr = am;
		tree = archMgr.getDataIdentTree();
		resetInput();
	}

	/** @see de.bsvrz.sys.funclib.losb.util.cmdinterface.Command#execute() */
	public void execute() throws Exception {
		resetInput();

		println("");
		printlnPlain("    Bedienung:");
		printlnPlain("      <Zahl>           Auswahl");
		printlnPlain("      obj,atg,asp,sv   Direktauswahl (ID oder PID)");
		printlnPlain("      q                Abbrechen");
		printlnPlain("      ^                Neue Auswahl");
		printBlank(2);
		printlnPlain("Weiter mit [Enter]");

		// Datenverteiler initialisieren
		model = archMgr.getDataModel();
		if(model == null) {
			printlnPlain("Abbruch (Datenverteiler nicht bereit).");
			stopCmd = true;
		}

		// Vorselektion (bersprungen mit [Enter]) setzt obj, atg, asp, simVar
		String input = getUserInput();
		try {
			getDirectNavigation(input);
		}
		catch(Exception confEx) {
			printlnPlain("Datenidentifikation ist teilweise ungueltig,");
		}

		// Objekt auswhlen
		if(!stopCmd && obj == -1) {
			objects = tree.getObjects(model);
			obj = getObject(objects);
			if(obj == -1) stopCmd = true;
		}

		// Attributgruppe auswhlen
		if(!stopCmd && atg == -1) {
			atgs = tree.getAtgs(objects[obj].getId(), model);
			atg = getAtg(atgs);
			if(atg == -1) stopCmd = true;
		}

		// Aspekt auswhlen
		if(!stopCmd && asp == -1) {
			asps = tree.getAsps(objects[obj].getId(), atgs[atg].getId(), model);
			asp = getAsp(asps);
			if(asp == -1) stopCmd = true;
		}

		// Simulationsvariante auswhlen
		if(!stopCmd && simVar == -2) {
			simVars = tree.getSimVars(objects[obj].getId(), atgs[atg].getId(), asps[asp].getId());
			simVar = getSimVar(simVars);
			if(simVar == -2) stopCmd = true;
		}

		// Informationen zum Objekt in Klartext
		Vector<String[]> info = new Vector<String[]>(4);
		if(!stopCmd) {
			info.add(new String[]{objects[obj].getName(), objects[obj].getPid(), "" + objects[obj].getId()});
			info.add(new String[]{atgs[atg].getName(), atgs[atg].getPid(), "" + atgs[atg].getId()});
			info.add(new String[]{asps[asp].getName(), asps[asp].getPid(), "" + asps[asp].getId()});
			info.add(new String[]{"" + simVar});
		}

		// Informationen fr die Datenidentifikation ausgeben
		if(!stopCmd) printInfo(objects[obj].getId(), atgs[atg].getId(), asps[asp].getId(), simVar, info);
	}

	/** Felder fr Objekte, Attributgruppen, Aspekte, SimulationsVarianten zurcksetzen */
	public void resetInput() {
		stopCmd = false;
		objects = null;
		obj = -1;
		atgs = null;
		atg = -1;
		asps = null;
		asp = -1;
		simVars = null;
		simVar = -2;
	}

	/**
	 * Direktnavigation mit kommaseparierten Werten
	 *
	 * @param input
	 *
	 * @throws Exception
	 */
	public void getDirectNavigation(String input) throws Exception {
		if(input != null) {
			String[] di = input.split(",");
			// Objekt vorselektieren
			if(!di[0].equals("")) {
				long objId = 0;
				try {
					objId = Long.parseLong(di[0]);
				}
				catch(NumberFormatException nfEx) {
					objId = model.getObject(di[0]).getId();
				}
				if(objId != 0) {
					objects = tree.getObjects(model);
					for(int i = 0; i < objects.length; i++) {
						if(objects[i].getId() == objId) obj = i;
					}
				}
			}
			// Attributgruppe vorselektieren
			if(di.length > 1 && !di[1].equals("")) {
				long atgId = -10;
				try {
					atgId = Long.parseLong(di[1]);
				}
				catch(NumberFormatException nfEx) {
					atgId = model.getObject(di[1]).getId();
				}
				if(atgId != -10) {
					atgs = tree.getAtgs(objects[obj].getId(), model);
					for(int i = 0; i < atgs.length; i++) {
						if(atgs[i].getId() == atgId) atg = i;
					}
				}
			}
			// Aspekt voreselektieren
			if(di.length > 2 && !di[2].equals("")) {
				long aspId = -10;
				try {
					aspId = Long.parseLong(di[2]);
				}
				catch(NumberFormatException nfEx) {
					aspId = model.getObject(di[2]).getId();
				}
				if(aspId != -10) {
					asps = tree.getAsps(objects[obj].getId(), atgs[atg].getId(), model);
					for(int i = 0; i < asps.length; i++) {
						if(asps[i].getId() == aspId) asp = i;
					}
				}
			}
			// Simulationsvariante vorselektieren
			if(di.length > 3) {
				try {
					simVar = Integer.parseInt(di[3]);
				}
				catch(NumberFormatException nfEx) {
					if(di[3].equals("")) simVar = 0;
				}
				if(simVar != -2) {
					simVars = tree.getSimVars(objects[obj].getId(), atgs[atg].getId(), asps[asp].getId());
					for(int i = 0; i < simVars.length; i++) {
						if(simVars[i] == simVar) simVar = i;
					}
				}
			}
		}
	}


	/**
	 * Objekt aus allen mglichen des DataIdentTree auswhlen
	 *
	 * @param objects
	 *
	 * @return die Id des Objekts, -1 bei Abbruch durch User
	 *
	 * @throws Exception
	 */
	public int getObject(SystemObject[] objects) throws Exception {
		objects = sortArray(objects);
		printlnPlain("Objekt waehlen (" + objects.length + "):");
		int obj = -1;
		int page = 0;
		for(int i = 0; i < objects.length && !stopCmd; i++) {
			String objOut = "  " + i + ":\t" + objects[i].getId() + ", " + objects[i].getPid() + ", " + objects[i].getName();
			objOut = (objOut.length() > maxPerLine ? objOut.substring(0, maxPerLine - 3) + "..." : objOut);
			printlnPlain(objOut);
			if(((i + 1) % linesPerPage) == 0 || (i + 1) == objects.length) {
				page++;
				if((1 + ((objects.length - 1) / linesPerPage)) > 1) {
					printlnPlain("Seite " + page + " von " + (1 + ((objects.length - 1) / linesPerPage)) + PAGER);
				}
				String input = getUserInput();
				if(input == null) {
					break;
				}
				else if(input.equals("<")) {
					page = (i > linesPerPage) ? page - 2 : 0;
					i = (i > linesPerPage) ? (i - 2 * linesPerPage) : -1;
				}
				else if(input.equals(">") || (input.equals(""))) {
					if(objects.length == 1) obj = 0;
				}
				else if(input.equals("^")) {
					execute();
					stopCmd = true;
				}
				else {
					try {
						obj = Integer.parseInt(input);
						if(obj >= objects.length) obj = getObject(objects);
						break;
					}
					catch(NumberFormatException nfEx) {
						/* noop */
					}
				}
			}
			if(i == objects.length - 1 && obj == -1) {
				i = 0;
				page = 0;
			}
		}
		return obj;
	}

	/**
	 * Attributgruppe aus allen mglichen fr das zuvor ausgewhlte Objekt whlen
	 *
	 * @param atgs
	 *
	 * @return die Id der Attributgruppe, -1 bei Abbruch durch User
	 *
	 * @throws Exception
	 */
	public int getAtg(AttributeGroup[] atgs) throws Exception {
		atgs = (AttributeGroup[])sortArray((SystemObject[])atgs);
		printlnPlain("Attributgruppe waehlen (" + atgs.length + "):");
		int atg = -1;
		int page = 0;
		for(int i = 0; i < atgs.length && !stopCmd; i++) {
			String objOut = "  " + i + ":\t" + atgs[i].getId() + ", " + atgs[i].getPid() + ", " + atgs[i].getName();
			objOut = (objOut.length() > maxPerLine ? objOut.substring(0, maxPerLine - 3) + "..." : objOut);
			printlnPlain(objOut);
			if(((i + 1) % linesPerPage) == 0 || (i + 1) == atgs.length) {
				page++;
				if((1 + ((atgs.length - 1) / linesPerPage)) > 1) printlnPlain("Seite " + page + " von " + (1 + ((atgs.length - 1) / linesPerPage)) + PAGER);
				String input = getUserInput();
				if(input == null) {
					break;
				}
				else if(input.equals("<")) {
					page = (i > linesPerPage) ? page - 2 : 0;
					i = (i > linesPerPage) ? (i - 2 * linesPerPage) : -1;
				}
				else if(input.equals(">") || (input.equals(""))) {
					if(atgs.length == 1) atg = 0;
				}
				else if(input.equals("^")) {
					execute();
					stopCmd = true;
				}
				else {
					try {
						atg = Integer.parseInt(input);
						if(atg >= atgs.length) atg = getAtg(atgs);
						break;
					}
					catch(NumberFormatException nfEx) {
						/* noop */
					}
				}
			}
			if(i == atgs.length - 1 && atg == -1) {
				i = 0;
				page = 0;
			}
		}
		return atg;
	}

	/**
	 * Aspekt fr das zuvor gewhlte Objekt und Attributgruppe whlen
	 *
	 * @param asps mgliche Aspekte
	 *
	 * @return die Id des gewhlten Aspekts
	 *
	 * @throws Exception
	 */
	public int getAsp(Aspect[] asps) throws Exception {
		asps = (Aspect[])sortArray((SystemObject[])asps);
		printlnPlain("Aspekt waehlen (" + asps.length + "):");
		int asp = -1;
		int page = 0;
		for(int i = 0; i < asps.length && !stopCmd; i++) {
			String objOut = "  " + i + ":\t" + asps[i].getId() + ", " + asps[i].getPid() + ", " + asps[i].getName();
			objOut = (objOut.length() > maxPerLine ? objOut.substring(0, maxPerLine - 3) + "..." : objOut);
			printlnPlain(objOut);
			if(((i + 1) % linesPerPage) == 0 || (i + 1) == asps.length) {
				page++;
				if((1 + ((asps.length - 1) / linesPerPage)) > 1) printlnPlain("Seite " + page + " von " + (1 + ((asps.length - 1) / linesPerPage)) + PAGER);
				String input = getUserInput();
				if(input == null) {
					break;
				}
				else if(input.equals("<")) {
					page = (i > linesPerPage) ? page - 2 : 0;
					i = (i > linesPerPage) ? (i - 2 * linesPerPage) : -1;
				}
				else if(input.equals(">") || (input.equals(""))) {
					if(asps.length == 1) asp = 0;
				}
				else if(input.equals("^")) {
					execute();
					stopCmd = true;
				}
				else {
					try {
						asp = Integer.parseInt(input);
						if(asp >= asps.length) asp = getAsp(asps);
						break;
					}
					catch(NumberFormatException nfEx) {
						/* noop */
					}
				}
			}
			if(i == asps.length - 1 && asp == -1) {
				i = -1;
				page = 0;
			}
		}
		return asp;
	}

	/**
	 * Simulationsvariante fr die ansonsten spezifizierte Datenidentifikation whlen
	 *
	 * @param simVars
	 *
	 * @return die Simulationsvariante, -1 bei Abbruch durch User
	 */
	public int getSimVar(int[] simVars) throws Exception {
		printlnPlain("Simulationsvariante waehlen (" + simVars.length + "):");
		int simVar = -2;
		int page = 0;
		for(int i = 0; i < simVars.length && !stopCmd; i++) {
			printlnPlain("  " + i + ":\tSimVar " + simVars[i]);
			if(((i + 1) % linesPerPage) == 0 || (i + 1) == simVars.length) {
				page++;
				if((1 + ((simVars.length - 1) / linesPerPage)) > 1) {
					printlnPlain("Seite " + page + " von " + (1 + ((simVars.length - 1) / linesPerPage)) + PAGER);
				}
				String input = getUserInput();
				if(input == null) {
					break;
				}
				else if(input.equals("<")) {
					page = (i > linesPerPage) ? page - 2 : 0;
					i = (i > linesPerPage) ? (i - 2 * linesPerPage) : -1;
				}
				else if(input.equals(">") || (input.equals(""))) {
					if(simVars.length == 1) simVar = 0;
				}
				else if(input.equals("^")) {
					execute();
					stopCmd = true;
				}
				else {
					try {
						simVar = Integer.parseInt(input);
						if(simVar >= simVars.length) simVar = getSimVar(simVars);
						break;
					}
					catch(NumberFormatException nfEx) {
						/* noop */
					}
				}
			}
			if(i == simVars.length - 1 && simVar == -2) {
				i = 0;
				page = 0;
			}
		}
		return simVar;
	}

	/**
	 * gibt das ausgewaehlte Objekt auf StdOut aus
	 *
	 * @param objId
	 * @param atgId
	 * @param aspId
	 * @param simVar
	 */
	public void printInfo(long objId, long atgId, long aspId, int simVar, Vector<String[]> info) throws Exception {
		int p = 20;
		printlnPlain("Datenidentifikation:");
		try {
			// Infos zur Datenidentifikation
			printlnPlain(Util.sr("  Objekt", p) + info.get(0)[0]);
			printlnPlain(Util.sr("", p) + info.get(0)[2]);
			printlnPlain(Util.sr("", p) + info.get(0)[1]);
			printlnPlain(Util.sr("  AttGrp", p) + info.get(1)[0]);
			printlnPlain(Util.sr("", p) + info.get(1)[2]);
			printlnPlain(Util.sr("", p) + info.get(1)[1]);
			printlnPlain(Util.sr("  Aspekt", p) + info.get(2)[0]);
			printlnPlain(Util.sr("", p) + info.get(2)[2]);
			printlnPlain(Util.sr("", p) + info.get(2)[1]);
			printlnPlain(Util.sr("  SimVar", p) + info.get(3)[0]);

			// Infos zum din
			DataIdentNode din = tree.getDataIdentNode(objId, atgId, aspId, simVar);
			printlnPlain(Util.sr("  Archivieren", p) + (din.arSParamIsArchivieren() ? "ja" : "nein"));
			printlnPlain(Util.sr("  Vorhalten", p) + Util.relTimestr(din.arSParamGetVorhalten()));
			if(din.arSParamIsNachfordern()) {
				printlnPlain(Util.sr("  Nachfordern", p) + din.arSParamGetNachfordern(0));
				for(int i = 1; i < din.arSParamGetAnzNachfordern(); i++) {
					printlnPlain(Util.sr("", p) + din.arSParamGetNachfordern(i));
				}
			}
			else {
				printlnPlain(Util.sr("  Nachfordern", p) + "nein");
			}

			if(din.arSParamIsQuittieren()) {
				String quitAspPid = model.getObject(din.arSParamGetQuittieren()).getPid();
				printlnPlain(Util.sr("  Quittieren", p) + din.arSParamGetQuittieren() + " (" + quitAspPid + ")");
				if(din.arSParamIsArchivieren()) printlnPlain(Util.sr("    gueltig", p) + (din.arSParamIsQuittierenValid() ? "ja" : "nein"));
			}
			else {
				printlnPlain(Util.sr("  Quittieren", p) + "nein");
			}

			printlnPlain(Util.sr("  Sichern", p) + (din.arSParamIsSichern() ? "ja" : "nein"));
			printlnPlain(Util.sr("  Parametriert", p) + (din.isArSParameterized() ? "ja" : "nein"));
			printlnPlain("");
			printlnPlain("Beenden mit [Enter], Neustart mit [^].");
			if(readln().equalsIgnoreCase("^")) execute();
		}
		catch(Exception e) {
			printlnPlain("Fehler: (" + e.getMessage() + ")");
		}
	}


	/**
	 * Sortiert eine Array von Objekten nach der Pid mit Quicksort
	 *
	 * @param in
	 *
	 * @return
	 *
	 * @throws Exception
	 */
	@SuppressWarnings({"unchecked"})
	public SystemObject[] sortArray(SystemObject[] in) throws Exception {
		Arrays.sort(
				in, new Comparator() {
			public int compare(Object a, Object b) {
				return ((SystemObject)a).getPid().compareToIgnoreCase(((SystemObject)b).getPid());
			}
		}
		);
		return in;
	}


	/**
	 * liest die benutzereingabe oder setzt das Flag zum Abbrechen, falls "e" eingegeben wird
	 *
	 * @return the inputString
	 *
	 * @throws Exception
	 */
	public String getUserInput() throws Exception {
		String input = readln();
		if(input.equals("q")) {
			stopCmd = true;
			return null;
		}
		return input;
	}


	/**
	 * prints blank lines
	 *
	 * @param count number of blank lines
	 */
	public void printBlank(int count) throws Exception {
		for(int i = 0; i < count; i++) {
			printlnPlain("");
        }
    }

}
