/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.ars.ars.mgmt.simulation;

import de.bsvrz.dav.daf.main.ResultData;

/**
 * Klasse fr Lsch- und Datenanmeldungsauftrge. Diese Auftrge werden von {@link de.bsvrz.ars.ars.mgmt.simulation.SimulationAutomaton} veranlasst.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision: 6419 $ / $Date: 2009-03-10 23:15:57 +0100 (Di, 10 Mrz 2009) $ / ($Author: rs $)
 */
public class SimulationResultData extends ResultData {

	/** Simulations-Automat. */
	private SimulationAutomaton dfa;

	/**
	 * Erzeugt das Objekt. Setzt alle Werte des {@link ResultData} Objekts auf null.
	 *
	 * @param dfa Simulation.
	 *
	 * @see ResultData#ResultData(SystemObject,DataDescription,long,Data)
	 */
	public SimulationResultData(SimulationAutomaton dfa) {
		super(null, null, -1L, null);
		this.dfa = dfa;
	}

	/**
	 * Erzeugt das Objekt und bernimmt die Werte aus dem bergebenen <code>ResultData</code> Objekt.
	 *
	 * @param dfa        Simulation
	 * @param resultData Ausgangsdaten.
	 */
	public SimulationResultData(SimulationAutomaton dfa, ResultData resultData) {
		super(
				resultData.getObject(),
				resultData.getDataDescription(),
				resultData.getDataTime(),
				resultData.getData(),
				resultData.isDelayedData(),
				resultData.getDataState()
		);
		this.dfa = dfa;
	}

	/**
	 * Wird vom Lsch-Task aufgerufen, wenn der Auftrag durchgefhrt wurde.
	 *
	 * @param successful Zeigt an ob der Auftrag erfolgreich durchgefhrt wurde.
	 */
	public void dataDeleted(boolean successful) {
		dfa.dataDeleted(successful);
	}

	/**
	 * Wird vom Anmeldungs-Task aufgerufen, nachdem sich das Archivsystem auf die Simulationsdaten angemeldet hat.
	 *
	 * @param successful Zeigt an ob der Auftrag erfolgreich durchgefhrt wurde.
	 */
	public void doneSubscribing(boolean successful) {
		dfa.finishInitState(successful);
	}

	/** @return Liefert die Simulationsvariante zurck, die gelscht werden soll. */
	public short getSimVar() {
		return dfa.getSimVar();
	}
}
